module.exports = function formatWebpackMessage(message) {
	let lines = message.split('\n')

	// Remove webpack-specific loader notation from filename.
	// Before:
	// ./~/css-loader!./~/postcss-loader!./src/App.css
	// After:
	// ./src/App.css
	if (lines[0].lastIndexOf('!') !== -1) {
		lines[0] = lines[0].substr(lines[0].lastIndexOf('!') + 1);
	}

	lines = lines.filter((line) => {
		// Webpack adds a list of entry points to warning messages:
		//  @ ./src/index.js
		//  @ multi react-scripts/~/react-dev-utils/webpackHotDevClient.js ...
		// It is misleading (and unrelated to the warnings) so we clean it up.
		// It is only useful for syntax errors but we have beautiful frames for them.
		return line.indexOf(' @ ') !== 0;
	})

	// line #0 is filename
	// line #1 is the main error message
	if (!lines[0] || !lines[1]) {
		return lines.join('\n')
	}

	// Cleans up verbose "module not found" messages for files and packages.
	if (lines[1].indexOf('Module not found: ') === 0) {
		lines = [
			lines[0],
			// Clean up message because "Module not found: " is descriptive enough.
			lines[1]
				.replace("Cannot resolve 'file' or 'directory' ", '')
				.replace('Cannot resolve module ', '')
				.replace('Error: ', '')
		]
	}

	// Cleans up syntax error messages.
	if (lines[1].indexOf('Module build failed: ') === 0) {
		lines[1] = lines[1].replace(
			'Module build failed: SyntaxError:',
			'Syntax error:'
		)
	}

	// Clean up export errors.
	// TODO: we should really send a PR to Webpack for this.
	let exportError = /\s*(.+?)\s*(")?export '(.+?)' was not found in '(.+?)'/;
	if (lines[1].match(exportError)) {
		lines[1] = lines[1].replace(
			exportError,
			"$1 '$4' does not contain an export named '$3'."
		)
	}

	// Reassemble the message.
	message = lines.join('\n')
	// Internal stacks are generally useless so we strip them... with the
	// exception of stacks containing `webpack:` because they're normally
	// from user code generated by WebPack. For more information see
	// https://github.com/facebookincubator/create-react-app/pull/1050
	message = message.replace(
		/^\s*at\s((?!webpack:).)*:\d+:\d+[\s)]*(\n|$)/gm,
		''
	) // at ... ...:x:y

	return message.trim()
}

